replacement_words = {
        "-\n": '',
        "\n-": '',
        "\n\n": " ",
        "\n": " ",
        "1-D": "oneD",
        "1D": "oneD",
        "2-D": "twoD",
        "2D": "twoD",
        "3-D": "threeD",
        "3D": "threeD",
        "non uniform": "non-uniform",
        'process': '',
        'condition': '',
        'study': '',
        'method': '',
        'paper': '',
        'figure': '',
        'author': '',
        'CO2': 'COtwo',
        }

parsing_corrections={
        ' uid': ' fluid',
        ' uids': ' fluids',
        ' ow' : ' flow',
        ' ows': ' flows',
        ' ood': ' flood',
        ' oods': ' floods',
        ' ield': ' field',
        ' eld': ' field',
        ' ields': ' fields',
        ' inite': ' finite',
        'inuence' : 'influence',
        ' oodplain' : 'floodplain',
        'outow': 'outflow',
        ' irst': 'first',
}

reported_corrections = {
            ' - ': '-',
            ' – ': '–',
            ' , ': ', ',
            '( ': '(',
            ' )': ')',
            " ' ": "'",
            " / ": "/",
            ' these ': " the ",
            ' our ': ' their ',
            "the present ": " their ",
            "and and": 'and',
            'nonuniform': 'non-uniform',
            'also' : '',
            "In this ": 'in their ',
            ' this ': " their ",
            'in addition': '',
            '.we ': ".They ",
            ',we ': ",they ",
            ' we ': " they ",
            " us ": " them ",
            'Fig.': "",
            "Figure": "",
            '(i':'',
            '(e':"",
            ' above': '',
            ' cited ': ' ',
        }