fields_codes_elsevier={'Chemistry':['1500-1508','1600-1606'],
           'Artificial Intelligence': ['1702','1707','1711'],
           'Life and Health Science': ['1100-1111','1502','1300-1315', '1911','2204','2307','2400-2406','2700-2748', '2800-2809','2900-2923','3000-3005','3400-3404','3500-3506','3600-3616'],
           'Environmental Science and Renewable Energy': ['1900-1910','1913','2103','2105','2300-2306','2312','2309-2312'],
           'Material Science': ['2211','2500-2508'],
           'Physics': ['1506','1507','3100-3110','1607','2100-2102','2104'],
           'Mathematics': ['1703','2600-2614'],
           'Social Sciences': ['1200-1213','1400-1410','1800-1804','2000-2003','2308','3200-3207','3300-3322'],
           'Computer Science': ['1700-1712'],
           'Engineering': ['1500-1502', '1909', '2102','2104','2200-2216', '1711','1603'],
           'Economics and Finance': ['1804','2000-2003'],
           'Multidisciplinary':['1000']}

all_fields=['Economics and Finance',
 'Chemistry',
 'Artificial Intelligence',
 'Life and Health Science',
 'Material Science',
 'Physics',
 'Mathematics',
 'Social Sciences',
 'Computer Science',
 'Engineering',
 'Environmental Science and Renewable Energy',]


arxiv_fields_abbrevs = {'cs': 'Computer Science',
 'econ': 'Economics',
 'eess': 'Electrical Engineering and Systems Science',
 'math': 'Mathematics',
 'astro-ph': 'Astrophysics',
 'cond-mat': 'Condensed Matter',
 'gr-qc': 'General Relativity and Quantum Cosmology',
 'hep-ex': 'High Energy Physics - Experiment',
 'hep-lat': 'High Energy Physics - Lattice',
 'hep-ph': 'High Energy Physics - Phenomenology',
 'hep-th': 'High Energy Physics - Theory',
 'math-ph': 'Mathematical Physic',
 'nlin' : 'Nonlinear Sciences',
 'nucl-ex': 'Nuclear Experiment',
 'nucl-th': 'Nuclear Theory',
 'physics': 'physics',
 'quant-ph': 'Quantum Physics',
 'q-bio': 'Quantitative Biology',
 'q-fin': 'Quantitative Finance',
 'stat': 'Statistics'}

arxiv_cs_categories =  {'cs.AI' : 'Artificial Intelligence',
 'cs.AR' : 'Hardware Architecture',
 'cs.CC' : 'Computational Complexity',
 'cs.CE' : 'Computational Engineering, Finance, and Science',
 'cs.CG' : 'Computational Geometry',
 'cs.CL' : 'Computation and Language',
 'cs.CR' : 'Cryptography and Security',
 'cs.CV' : 'Computer Vision and Pattern Recognition',
 'cs.CY' : 'Computers and Society',
 'cs.DB' : 'Databases',
 'cs.DC' : 'Distributed, Parallel, and Cluster Computing',
 'cs.DL' : 'Digital Libraries',
 'cs.DM' : 'Discrete Mathematics',
 'cs.DS' : 'Data Structures and Algorithms',
 'cs.ET' : 'Emerging Technologies',
 'cs.FL' : 'Formal Languages and Automata Theory',
 'cs.GL' : 'General Literature',
 'cs.GR' : 'Graphics',
 'cs.GT' : 'Computer Science and Game Theory',
 'cs.HC' : 'Human-Computer Interaction',
 'cs.IR' : 'Information Retrieval',
 'cs.IT' : 'Information Theory',
 'cs.LG' : 'Machine Learning',
 'cs.LO' : 'Logic in Computer Science',
 'cs.MA' : 'Multiagent Systems',
 'cs.MM' : 'Multimedia',
 'cs.MS' : 'Mathematical Software',
 'cs.NA' : 'Numerical Analysis',
 'cs.NE' : 'Neural and Evolutionary Computing',
 'cs.NI' : 'Networking and Internet Architecture',
 'cs.OH' : 'Other Computer Science',
 'cs.OS' : 'Operating Systems',
 'cs.PF' : 'Performance',
 'cs.PL' : 'Programming Languages',
 'cs.RO' : 'Robotics',
 'cs.SC' : 'Symbolic Computation',
 'cs.SD' : 'Sound',
 'cs.SE' : 'Software Engineering',
 'cs.SI' : 'Social and Information Networks',
 'cs.SY' : 'Systems and Control'}
arxiv_econ_categories={'econ.EM': 'Econometrics',
 'econ.GN': 'General Economics',
 'econ.TH': 'Theoretical Economics'}
arxiv_eess_categories = {'eess.AS': 'Audio and Speech Processing',
'eess.IV': 'Image and Video Processing',
'eess.SP': 'Signal Processing',
'eess.SY': 'Systems and Control'}
arxiv_math_categories = {'math.AC': 'Commutative Algebra',
 'math.AG': 'Algebraic Geometry',
 'math.AP': 'Analysis of PDEs',
 'math.AT': 'Algebraic Topology',
 'math.CA': 'Classical Analysis and ODEs',
 'math.CO': 'Combinatorics',
 'math.CT': 'Category Theory',
 'math.CV': 'Complex Variables',
 'math.DG': 'Differential Geometry',
 'math.DS': 'Dynamical Systems',
 'math.FA': 'Functional Analysis',
 'math.GM': 'General Mathematics',
 'math.GN': 'General Topology',
 'math.GR': 'Group Theory',
 'math.GT': 'Geometric Topology',
 'math.HO': 'History and Overview',
 'math.IT': 'Information Theory',
 'math.KT': 'K-Theory and Homology',
 'math.LO': 'Logic',
 'math.MG': 'Metric Geometry',
 'math.MP': 'Mathematical Physics',
 'math.NA': 'Numerical Analysis',
 'math.NT': 'Number Theory',
 'math.OA': 'Operator Algebras',
 'math.OC': 'Optimization and Control',
 'math.PR': 'Probability',
 'math.QA': 'Quantum Algebra',
 'math.RA': 'Rings and Algebras',
 'math.RT': 'Representation Theory',
 'math.SG': 'Symplectic Geometry',
 'math.SP': 'Spectral Theory',
 'math.ST': 'Statistics Theory'}
arxiv_astro_categories = { 'astro-ph.CO': 'Cosmology and Nongalactic Astrophysics',
 'astro-ph.EP': 'Earth and Planetary Astrophysics',
 'astro-ph.GA': 'Astrophysics of Galaxies',
 'astro-ph.HE': 'High Energy Astrophysical Phenomena',
 'astro-ph.IM': 'Instrumentation and Methods for Astrophysics',
 'astro-ph.SR': 'Solar and Stellar Astrophysics'}
arxiv_cond_categories = {'cond-mat.dis-nn' : 'Disordered Systems and Neural Networks',
'cond-mat.mes-hall' : 'Mesoscale and Nanoscale Physics',
'cond-mat.mtrl-sci' : 'Materials Science',
'cond-mat.other' : 'Other Condensed Matter',
'cond-mat.quant-gas' : 'Quantum Gases',
'cond-mat.soft' : 'Soft Condensed Matter',
'cond-mat.stat-mech' : 'Statistical Mechanics',
'cond-mat.str-el' : 'Strongly Correlated Electrons',
'cond-mat.supr-con' : 'Superconductivity'}
arxiv_grqc_categories = {'gr-qc': 'General Relativity and Quantum Cosmology'}
arxiv_hepex_categories = {'hep-ex': 'High Energy Physics - Experiment'}
arxiv_heplat_categories = {'hep-lat' : 'High Energy Physics - Lattice'}
arxiv_hepph_categories = {'hep-ph': 'High Energy Physics - Phenomenology'}
arxiv_hepth_categories = {'hep-th' : 'High Energy Physics - Theory'}
arxiv_mathph_categories = {'math-ph' : 'Mathematical Physics'}
arxiv_nonlinear_categories = {'nlin.AO' : 'Adaptation and Self-Organizing Systems',
 'nlin.CD' : 'Chaotic Dynamics',
 'nlin.CG' : 'Cellular Automata and Lattice Gases',
 'nlin.PS' : 'Pattern Formation and Solitons',
 'nlin.SI' : 'Exactly Solvable and Integrable Systems'}
arxiv_nuclex_categories = {'nucl-ex': 'Nuclear Experiment'}
arxiv_nuclth_categories = {'nucl-th': 'Nuclear Theory'}
arxiv_physics_categories = {'physics.acc-ph' : 'Accelerator Physics',
'physics.ao-ph' : 'Atmospheric and Oceanic Physics',
'physics.app-ph' : 'Applied Physics',
'physics.atm-clus' : 'Atomic and Molecular Clusters',
'physics.atom-ph' : 'Atomic Physics',
'physics.bio-ph' : 'Biological Physics',
'physics.chem-ph' : 'Chemical Physics',
'physics.class-ph' : 'Classical Physics',
'physics.comp-ph' : 'Computational Physics',
'physics.data-an' : 'Data Analysis, Statistics and Probability',
'physics.ed-ph' : 'Physics Education',
'physics.flu-dyn' : 'Fluid Dynamics',
'physics.gen-ph' : 'General Physics',
'physics.geo-ph' : 'Geophysics',
'physics.hist-ph' : 'History and Philosophy of Physics',
'physics.ins-det' : 'Instrumentation and Detectors',
'physics.med-ph' : 'Medical Physics',
'physics.optics' : 'Optics',
'physics.plasm-ph' : 'Plasma Physics',
'physics.pop-ph' : 'Popular Physics',
'physics.soc-ph' : 'Physics and Society',
'physics.space-ph' : 'Space Physics'}
arxiv_quantph_categories = {'quant-ph': 'Quantum Physics'}
arxiv_qbio_categories = {'q-bio.BM' : 'Biomolecules',
'q-bio.CB' : 'Cell Behavior',
'q-bio.GN' : 'Genomics',
'q-bio.MN' : 'Molecular Networks',
'q-bio.NC' : 'Neurons and Cognition',
'q-bio.OT' : 'Other Quantitative Biology',
'q-bio.PE' : 'Populations and Evolution',
'q-bio.QM' : 'Quantitative Methods',
'q-bio.SC' : 'Subcellular Processes',
'q-bio.TO' : 'Tissues and Organs'}
arxiv_qfin_categories = {'q-fin.CP': 'Computational Finance',
'q-fin.EC': 'Economics',
'q-fin.GN': 'General Finance',
'q-fin.MF': 'Mathematical Finance',
'q-fin.PM': 'Portfolio Management',
'q-fin.PR': 'Pricing of Securities',
'q-fin.RM': 'Risk Management',
'q-fin.ST': 'Statistical Finance',
'q-fin.TR': 'Trading and Market Microstructure'}
arxiv_stat_categories = {'stat.AP': 'Applications',
'stat.CO': 'Computation',
'stat.ME': 'Methodology',
'stat.ML': 'Machine Learning',
'stat.OT': 'Other Statistics',
'stat.TH': 'Statistics Theory'}

arxiv_fields_categories = {'cs': arxiv_cs_categories,
                           'econ': arxiv_econ_categories,
                           'eess': arxiv_eess_categories,
                           'math': arxiv_math_categories,
                           'astro-ph': arxiv_astro_categories,
                           'cond-mat': arxiv_cond_categories,
                           'gr-qc': arxiv_grqc_categories,
                           'hep-ex': arxiv_hepex_categories,
                           'hep-lat': arxiv_heplat_categories,
                           'hep-ph': arxiv_hepph_categories,
                           'hep-th': arxiv_hepth_categories,
                           'math-ph': arxiv_mathph_categories,
                           'nlin': arxiv_nonlinear_categories,
                           'nucl-ex': arxiv_nuclex_categories,
                           'nucl-th': arxiv_nuclth_categories,
                           'physics': arxiv_physics_categories,
                           'quant-ph': arxiv_quantph_categories,
                           'q-bio': arxiv_qbio_categories,
                           'q-fin': arxiv_qfin_categories,
                           'stat': arxiv_stat_categories}
